﻿using Exam.ExamManages;
using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Web.Utilities;
using Exam.Questions;
using Hotline.Application.Exam.Extensions;
using Hotline.Share.Requests.Question;
using JiebaNet.Segmenter.Common;
using SqlSugar;
using System.Linq.Expressions;

namespace Hotline.Application.Exam.QueryExtensions.Questions
{
    public static class QuestionQueryExtesions
    {
        public static Expression<Func<Question, bool>> GetExpression(this QuestionPagedRequest questionPagedRequest)
        {
            Expression<Func<Question, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<Question>().AndIF(questionPagedRequest.DifficultyLevel.IsNotNull(), x => questionPagedRequest.DifficultyLevel == x.DifficultyLevel)
            .AndIF(questionPagedRequest.Title.IsNotNullOrEmpty(), x => x.Title.Contains(questionPagedRequest.Title))
            .ToExpression();
            return expression;
        }

        public static Expression<Func<QuestionTag, bool>> GetQuestionTagExpression(this QuestionPagedRequest questionPagedRequest)
        {
            Expression<Func<QuestionTag, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<QuestionTag>().AndIF(questionPagedRequest.TagIds.IsNotNullOrEmpty(), x => questionPagedRequest.TagIds.Contains(x.TagId))
            .ToExpression();
            return expression;
        }

        public static Expression<Func<ExamTag, bool>> GetExamTagExpression(this QuestionPagedRequest questionPagedRequest)
        {
            Expression<Func<ExamTag, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<ExamTag>().
            AndIF(questionPagedRequest.TagIds.IsNotNullOrEmpty(), x => questionPagedRequest.TagIds.Contains(x.Id))
            .ToExpression();

            return expression;
        }

        public static Expression<Func<QuestionKnowladge, bool>> GetQuestionKnowladgeExpression(this QuestionPagedRequest questionPagedRequest)
        {
            Expression<Func<QuestionKnowladge, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<QuestionKnowladge>().
            AndIF(questionPagedRequest.KnowladgeIds.IsNotNullOrEmpty(), x => questionPagedRequest.KnowladgeIds.Contains(x.QuestionId))
            .ToExpression();

            return expression;
        }
    }
}
