﻿using Exam.ExamManages;
using Exam.Infrastructure.Data.Entity;
using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Web.Utilities;
using Exam.Questions;
using Exam.TestPapers;
using Hotline.Application.Exam.Extensions;
using Hotline.Share.Enums.Exams;
using Hotline.Share.Requests.TestPaper;
using JiebaNet.Segmenter.Common;
using System.Linq.Expressions;

namespace Hotline.Application.Exam.QueryExtensions.TestPapers
{
    public static class TestPaperQueryExtensions
    {
        public static Expression<Func<TestPaper,bool>> GetExpression(this TestPaperPagedRequest testPaperPagedRequest)
        {
            Expression<Func<TestPaper, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<TestPaper>()
                .AndIF(testPaperPagedRequest.Name.IsNotNullOrEmpty(), x => x.Name.Contains(testPaperPagedRequest.Name))
                .AndIF(testPaperPagedRequest.Code.IsNotNullOrEmpty(), x => x.Code.Contains(testPaperPagedRequest.Code))
                .AndIF(testPaperPagedRequest.Status.IsNotNull(), x => x.Status == testPaperPagedRequest.Status)
                .AndIF(testPaperPagedRequest.ExamMode.IsNotNull(), x => x.Mode == testPaperPagedRequest.ExamMode)
                .AndIF(testPaperPagedRequest.ExamType.IsNotNull(), x => x.ExamType == testPaperPagedRequest.ExamType)
                .ToExpression();

            return expression;
        }

        public static Expression<Func<TestPaperRule, bool>> GetTestPaperRuleExpression(this EntityQueryRequest entityQueryRequest)
        {
            Expression<Func<TestPaperRule, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<TestPaperRule>()
                .AndIF(entityQueryRequest.Id.IsNotNullOrEmpty(), x => x.TestPaperId == entityQueryRequest.Id)
                .AndIF(entityQueryRequest.Ids.IsNotNullOrEmpty(), x => entityQueryRequest.Ids.Contains(x.TestPaperId))
                .ToExpression();

            return expression;
        }

        public static Expression<Func<TestPaper,bool>> GetTestPaperCountExpression(this TestPaperQuestionCountRequest testPaperQuestionCountRequest)
        {
            Expression<Func<TestPaper, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<TestPaper>()
               .AndIF(testPaperQuestionCountRequest.TestPaperId.IsNotNullOrEmpty(), x => x.Id == testPaperQuestionCountRequest.TestPaperId)
               .ToExpression();

            return expression;
        }

        public static Expression<Func<ExtractRule, bool>> GetExpression(this TestPaperQuestionCountRequest testPaperQuestionCountRequest)
        {
            Expression<Func<ExtractRule, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<ExtractRule>()
               .AndIF(testPaperQuestionCountRequest.ExtractRuleId.IsNotNullOrEmpty(), x => x.Id == testPaperQuestionCountRequest.ExtractRuleId)
               .ToExpression();

            return expression;
        }

        public static Expression<Func<TestPaperRule,bool>> GetTestPaperRuleExpression(this TestPaperQuestionCountRequest testPaperQuestionCountRequest)
        {
            Expression<Func<TestPaperRule, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<TestPaperRule>()
               .AndIF(testPaperQuestionCountRequest.TestPaperId.IsNotNullOrEmpty(), x => x.TestPaperId == testPaperQuestionCountRequest.TestPaperId)
               .ToExpression();

            return expression;
        }

        public static Expression<Func<TestPaperItem,bool>> GetTestPaperItemsExpression(this TestPaperQuestionCountRequest testPaperQuestionCountRequest)
        {
            Expression<Func<TestPaperItem, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<TestPaperItem>()
               .AndIF(testPaperQuestionCountRequest.TestPaperId.IsNotNullOrEmpty(), x => x.TestPaperId == testPaperQuestionCountRequest.TestPaperId)
               .ToExpression();

            return expression;
        }

        public static Expression<Func<QuestionTag,bool>> GetExpression(this TagQuestionCountRequest tagQuestionCountRequest)
        {
            Expression<Func<QuestionTag, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<QuestionTag>()
               .AndIF(tagQuestionCountRequest.TagIds.IsNotNullOrEmpty() && tagQuestionCountRequest.TagIds.Any(), x => tagQuestionCountRequest.TagIds.Contains(x.TagId))
               .ToExpression();

            return expression;
        }

        public static Expression<Func<Question,bool>> GetQuestionExpression(this TagQuestionCountRequest tagQuestionCountRequest)
        {
            Expression<Func<Question, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<Question>()
               .AndIF(tagQuestionCountRequest.QuestionType.IsNotNull(), x => tagQuestionCountRequest.QuestionType ==x.QuestionType)
               .AndIF(tagQuestionCountRequest.DifficultyLevel.IsNotNull(),x=> tagQuestionCountRequest.DifficultyLevel == x.DifficultyLevel)
               .AndIF(tagQuestionCountRequest.QuestionIds.IsNotNullOrEmpty(), x => !tagQuestionCountRequest.QuestionIds.Contains(x.Id))
               .ToExpression();

            return expression;
        }

        public static Expression<Func<QuestionTag, bool>> GetExpression(this TestPaperQuestionRequest testPaperQuestionRequest)
        {
            Expression<Func<QuestionTag, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<QuestionTag>()
               .AndIF(testPaperQuestionRequest.TagIds.IsNotNullOrEmpty(), x => testPaperQuestionRequest.TagIds.Contains(x.TagId))
               .ToExpression();

            return expression;
        }

        public static Expression<Func<Question, bool>> GetQuestionExpression(this TestPaperQuestionRequest testPaperQuestionRequest)
        {
            Expression<Func<Question, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<Question>()
               .AndIF(testPaperQuestionRequest.QuestionType.IsNotNull(), x => testPaperQuestionRequest.QuestionType == x.QuestionType)
               .AndIF(testPaperQuestionRequest.DifficultyLevel.IsNotNull(), x => testPaperQuestionRequest.DifficultyLevel == x.DifficultyLevel)
               .AndIF(testPaperQuestionRequest.QuestionIds.IsNotNullOrEmpty(), x => !testPaperQuestionRequest.QuestionIds.Contains(x.Id))
               .ToExpression();

            return expression;
        }
    }
}
