﻿using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Web.Utilities;
using Exam.Questions;
using Exam.TestPapers;
using Exam.Trains;
using Hotline.Exams.Trains;
using Hotline.Share.Requests.Exam;
using Hotline.Share.Requests.Train;
using JiebaNet.Segmenter.Common;
using System.Linq.Expressions;

namespace Hotline.Application.Exam.QueryExtensions.Trains
{
    public static class TrainRecordQueryExtensions
    {
        public static Expression<Func<TrainPlan,bool>> GetExpression(this TrainRecordPagedRequest trainRecordPagedRequest)
        {
            Expression<Func<TrainPlan, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<TrainPlan>()
                .AndIF(trainRecordPagedRequest.Code.IsNotNull(),x=>x.Code.Contains(trainRecordPagedRequest.Code))
                .AndIF(trainRecordPagedRequest.Name.IsNotNull(),x=>x.Name.Contains(trainRecordPagedRequest.Name))
                .AndIF(trainRecordPagedRequest.MinStartTime.IsNotNull(),x=>x.TrainStartTime>=trainRecordPagedRequest.MinStartTime)
                .AndIF(trainRecordPagedRequest.MaxStartTime.IsNotNull(),x=>x.TrainStartTime<=trainRecordPagedRequest.MaxStartTime)
                .AndIF(trainRecordPagedRequest.MinEndTime.IsNotNull(),x=>x.TrainEndTime>=trainRecordPagedRequest.MinEndTime)
                .AndIF(trainRecordPagedRequest.MaxEndTime.IsNotNull(),x=>x.TrainEndTime<=trainRecordPagedRequest.MaxEndTime)
                .ToExpression();

            return expression;
        }

        public static Expression<Func<TrainPractice, bool>> GetExpression(this TrainQuestionRequest trainQuestionRequest)
        {
            Expression<Func<TrainPractice, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<TrainPractice>()
                .AndIF(trainQuestionRequest.QuestionId.IsNotNullOrEmpty(), x => x.QuestionId == trainQuestionRequest.QuestionId).ToExpression();

            return expression;
        }

        public static Expression<Func<TrainPractice,bool>> GetExpression(this TrainPracticeRequest trainPracticeRequest)
        {
            Expression<Func<TrainPractice, bool>> expression = m => m.Id != null;

            expression = ExpressionableUtility.CreateExpression<TrainPractice>()
                .AndIF(trainPracticeRequest.TrainPracticeId.IsNotNullOrEmpty(), x => x.Id == trainPracticeRequest.TrainPracticeId).ToExpression();

            return expression;
        }
    }
}
