﻿using Exam.ExamManages;
using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Validation.Validation;
using Exam.Questions;
using FluentValidation;
using Hotline.Exams.ExamManages;
using Hotline.Repository.SqlSugar.Validate;

namespace Hotline.Repository.SqlSugar.Exam.Validators.ExamManages
{
    public class UserExamItemOptionsValidator: BaseValidator<UserExamItemOptions>
    {
        public UserExamItemOptionsValidator()
        {
            RuleSet(ValidatorTypeConstants.Create, () =>
            {
                BaseValidateRule();

                ValidateRuleWithAdd();
            });

            RuleSet(ValidatorTypeConstants.Modify, () =>
            {
                BaseValidateRule();

                ValidateRuleWithModify();
            });
        }

        protected override void BaseValidateRule()
        {
            base.BaseValidateRule();

            RuleFor(x => x.QuestionOptionId).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, typeof(QuestionOptions).GetDescription()));

            RuleFor(x => x.UserExamItemId).NotEmpty().WithMessage(x=>string.Format(ErrorMessage.IsRequired,typeof(UserExamItem).GetDescription()));
        }

        protected override void ValidateRuleWithAdd()
        {
            base.ValidateRuleWithAdd();
            RuleFor(m => m.CreationTime).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(UserExamItem.CreationTime))));
        }

        protected override void ValidateRuleWithModify()
        {
            base.ValidateRuleWithModify();
            RuleFor(m => m.LastModificationTime).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(UserExamItem.LastModificationTime))));

        }
    }
}
