using Exam.ExamManages;
using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Validation.Validation;
using Exam.Questions;
using FluentValidation;
using Hotline.Exams.Sourcewares;
using Hotline.Repository.SqlSugar.Validate;

namespace Exam.Repository.Sqlsugar.Validators.Questions
{
    public class TagQuestionValidator:BaseValidator<TagQuestion>
    {
        public TagQuestionValidator()
        {
            RuleSet(ValidatorTypeConstants.Create, () =>
            {
                BaseValidateRule();

                ValidateRuleWithAdd();
            });

            RuleSet(ValidatorTypeConstants.Modify, () =>
            {
                BaseValidateRule();

                ValidateRuleWithModify();
            });
        }

        protected override void BaseValidateRule()
        {
            base.BaseValidateRule();
            RuleFor(m => m.TagId).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, typeof(ExamTag).GetDescription()));
            RuleFor(m => m.RuleId).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, typeof(ExtractRule).GetDescription()));
        }

        protected override void ValidateRuleWithAdd()
        {
            base.ValidateRuleWithAdd();
            RuleFor(m => m.CreationTime).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(TagQuestion.CreationTime))));
        }

        protected override void ValidateRuleWithModify()
        {
            base.ValidateRuleWithModify();
            RuleFor(m => m.LastModificationTime).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(TagQuestion.LastModificationTime))));

        }

    }
}
