using Exam.ExamManages;
using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Validation.Validation;
using Exam.TestPapers;
using FluentValidation;
using Hotline.Repository.SqlSugar.Validate;

namespace Exam.Repository.Sqlsugar.Validators.TestPapers
{
    public class TestPaperRuleTagValidator:BaseValidator<TestPaperRuleTag>
    {
        public TestPaperRuleTagValidator()
        {
            RuleSet(ValidatorTypeConstants.Create, () =>
            {
                BaseValidateRule();

                ValidateRuleWithAdd();
            });

            RuleSet(ValidatorTypeConstants.Modify, () =>
            {
                BaseValidateRule();

                ValidateRuleWithModify();
            });
        }

        protected override void BaseValidateRule()
        {
            base.BaseValidateRule();

            RuleFor(m => m.TestPaperRuleId).NotEmpty().WithMessage(x=>string.Format(ErrorMessage.IsRequired,x.GetType().GetDescription(nameof(TestPaperRuleTag.TestPaperRuleId))));
            RuleFor(m => m.TagId).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, typeof(ExamTag).GetDescription()));
        }

        protected override void ValidateRuleWithAdd()
        {
            base.ValidateRuleWithAdd();
            RuleFor(m => m.CreationTime).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(TestPaperRuleTag.CreationTime))));
        }

        protected override void ValidateRuleWithModify()
        {
            base.ValidateRuleWithModify();
            RuleFor(m => m.LastModificationTime).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(TestPaperRuleTag.LastModificationTime))));

        }

    }
}
