using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Validation.Validation;
using Exam.TestPapers;
using FluentValidation;
using Hotline.Repository.SqlSugar.Validate;

namespace Exam.Repository.Sqlsugar.Validators.TestPapers
{
    public class TestPaperRuleValidator:BaseValidator<TestPaperRule>
    {
        public TestPaperRuleValidator()
        {
            RuleSet(ValidatorTypeConstants.Create, () =>
            {
                BaseValidateRule();

                ValidateRuleWithAdd();
            });

            RuleSet(ValidatorTypeConstants.Modify, () =>
            {
                BaseValidateRule();

                ValidateRuleWithModify();
            });
        }

        protected override void BaseValidateRule()
        {
            base.BaseValidateRule();

            RuleFor(m => m.TestPaperId).NotEmpty().WithMessage(x=>string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(TestPaperRule.TestPaperId))));

            RuleFor(m => m.QuestionType).NotNull().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(TestPaperRule.QuestionType))));

            RuleFor(m => m.Count).NotNull().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(TestPaperRule.QuestionType))));
        }

        protected override void ValidateRuleWithAdd()
        {
            base.ValidateRuleWithAdd();
            RuleFor(m => m.CreationTime).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(TestPaperRule.CreationTime))));
        }

        protected override void ValidateRuleWithModify()
        {
            base.ValidateRuleWithModify();
            RuleFor(m => m.LastModificationTime).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(TestPaperRule.LastModificationTime))));

        }

    }
}
