using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Validation.Validation;
using Exam.Trains;
using FluentValidation;
using Hotline.Repository.SqlSugar.Validate;
using Hotline.Users;

namespace Exam.Repository.Sqlsugar.Validators.Trains
{
    public class TrainRecordValidator:BaseValidator<TrainRecord>
    {
        public TrainRecordValidator()
        {
            RuleSet(ValidatorTypeConstants.Create, () =>
            {
                BaseValidateRule();

                ValidateRuleWithAdd();
            });

            RuleSet(ValidatorTypeConstants.Modify, () =>
            {
                BaseValidateRule();

                ValidateRuleWithModify();
            });
        }

        protected override void BaseValidateRule()
        {
            base.BaseValidateRule();

            RuleFor(m=>m.UserId).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, typeof(User).GetDescription()));
        }

        protected override void ValidateRuleWithAdd()
        {
            base.ValidateRuleWithAdd();
            RuleFor(m => m.CreationTime).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(TrainRecord.CreationTime))));
        }

        protected override void ValidateRuleWithModify()
        {
            base.ValidateRuleWithModify();
            RuleFor(m => m.LastModificationTime).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(TrainRecord.LastModificationTime))));

        }

    }
}
