﻿using Exam.Infrastructure.Extensions;
using Exam.Infrastructure.Validation.Validation;
using Exam.Questions;
using FluentValidation;
using Hotline.Share.Dtos.Questions;
using Hotline.Validators.Exams.Questions;

namespace Hotline.Validators.Exams
{
    public class UpdateQuestionDtoValidator:AbstractValidator<UpdateQuestionDto>
    {
        public UpdateQuestionDtoValidator()
        {
            RuleFor(m => m.Id).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(Question.Id))));
            RuleFor(m => m.Title).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(Question.Title))));
            RuleFor(m => m.DifficultyLevel).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(Question.DifficultyLevel))));
            RuleFor(m => m.FormalEnable).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(Question.FormalEnable))));
            RuleFor(m => m.SimulateEnable).NotEmpty().WithMessage(x => string.Format(ErrorMessage.IsRequired, x.GetType().GetDescription(nameof(Question.SimulateEnable))));

            RuleForEach(m => m.QuestionSourcewareDtos).SetValidator(new UpdateQuestionSourcewareDtoValidator());

            RuleForEach(m => m.QuestionKnowladgeDtos).SetValidator(new UpdateQuestionKnowladgeDtoValidator());

            RuleForEach(m => m.QuestionTagDtos).SetValidator(new UpdateQuestionTagDtoValidator());

            RuleForEach(m => m.QuestionOptionsDtos).SetValidator(new UpdateQuestionOptionsDtoValidator());
        }
    }
}
